@@ -26,9 +26,32 @@
 #include <gst/rtp/gstrtpbuffer.h>
 
 #include <gst/farsight/fs-conference-iface.h>
+#include <gst/farsight/fs-element-added-notifier.h>
 
 #include "check-threadsafe.h"
 
+#define BUFFER_COUNT 20
+
+GMutex *count_mutex;
+GCond *count_cond;
+guint buffer_count = 0;
+
+
+static void
+handoff_handler (GstElement *fakesink, GstBuffer *buffer, GstPad *pad,
+    gpointer user_data)
+{
+  g_mutex_lock (count_mutex);
+  buffer_count ++;
+
+  GST_LOG ("buffer %d", buffer_count);
+
+  if (buffer_count == BUFFER_COUNT)
+    g_cond_broadcast (count_cond);
+  ts_fail_unless (buffer_count <= BUFFER_COUNT);
+  g_mutex_unlock (count_mutex);
+}
+
 static void
 src_pad_added_cb (FsStream *self,
     GstPad   *pad,
@@ -39,7 +62,10 @@ src_pad_added_cb (FsStream *self,
   GstPad *sinkpad;
 
   sink = gst_element_factory_make ("fakesink", NULL);
-  g_object_set (sink, "sync", TRUE, NULL);
+  g_object_set (sink, "sync", TRUE,
+      "signal-handoffs", TRUE,
+      NULL);
+  g_signal_connect (sink, "handoff", G_CALLBACK (handoff_handler), NULL);
   fail_unless (gst_bin_add (GST_BIN (pipeline), sink));
   gst_element_set_state (sink, GST_STATE_PLAYING);
   sinkpad = gst_element_get_static_pad (sink, "sink");
@@ -49,7 +75,56 @@ src_pad_added_cb (FsStream *self,
   GST_DEBUG ("Pad added");
 }
 
-GST_START_TEST (test_rtprecv_no_config_data)
+static guint decoder_count = 0;
+
+static void
+element_added (FsElementAddedNotifier *notif, GstBin *bin, GstElement *element,
+    gpointer user_data)
+{
+  GstElementFactory *fact = gst_element_get_factory (element);
+
+  if (!fact)
+    return;
+
+  if (strcmp (gst_plugin_feature_get_name (GST_PLUGIN_FEATURE (fact)),
+              "theoradec"))
+    return;
+
+  ts_fail_unless (decoder_count == 0);
+  decoder_count++;
+}
+
+static void
+caps_changed (GstPad *pad, GParamSpec *spec, FsStream *stream)
+{
+  GstCaps *caps;
+  GstStructure *s;
+  FsCodec *codec;
+  GList *codecs;
+  const gchar *config;
+  GError *error = NULL;
+
+  g_object_get (pad, "caps", &caps, NULL);
+
+  if (!caps)
+    return;
+
+  s = gst_caps_get_structure (caps, 0);
+
+  codec = fs_codec_new (96, "THEORA", FS_MEDIA_TYPE_VIDEO, 90000);
+
+  config = gst_structure_get_string (s, "configuration");
+  if (config)
+    fs_codec_add_optional_parameter (codec, "configuration", config);
+
+  codecs = g_list_prepend (NULL, codec);
+  fail_unless (fs_stream_set_remote_codecs (stream, codecs, &error),
+      "Unable to set remote codec: %s",
+      error ? error->message : "UNKNOWN");
+  fs_codec_list_destroy (codecs);
+}
+
+GST_START_TEST (test_rtprecv_inband_config_data)
 {
   FsParticipant *participant = NULL;
   FsStream *stream = NULL;
@@ -64,9 +139,22 @@ GST_START_TEST (test_rtprecv_no_config_data)
   GstElement *conference;
   FsSession *session;
   GList *item;
+  GstPad *pad;
+  GstElement *pay;
+  FsElementAddedNotifier *notif;
+
+  count_mutex = g_mutex_new ();
+  count_cond = g_cond_new ();
+  buffer_count = 0;
+  decoder_count = 0;
 
   fspipeline = gst_pipeline_new (NULL);
 
+  notif = fs_element_added_notifier_new ();
+  fs_element_added_notifier_add (notif, GST_BIN (fspipeline));
+  g_signal_connect (notif, "element-added", G_CALLBACK (element_added), NULL);
+
+
   conference = gst_element_factory_make ("fsrtpconference", NULL);
 
   fail_unless (gst_bin_add (GST_BIN (fspipeline), conference));
@@ -77,8 +165,9 @@ GST_START_TEST (test_rtprecv_no_config_data)
     fail ("Error while creating new session (%d): %s",
         error->code, error->message);
   fail_if (session == NULL, "Could not make session, but no GError!");
+  g_object_set (session, "no-rtcp-timeout", 0, NULL);
 
-
+  g_object_get (session, "codecs-without-config", &codecs, NULL);
   for (item = codecs; item; item = item->next)
   {
     FsCodec *codec = item->data;
@@ -114,22 +203,21 @@ GST_START_TEST (test_rtprecv_no_config_data)
   g_signal_connect (stream, "src-pad-added",
       G_CALLBACK (src_pad_added_cb), fspipeline);
 
-  g_object_get (session, "codecs-without-config", &codecs, NULL);
-
-
   codecs = g_list_prepend (NULL, fs_codec_new (96, "THEORA",
           FS_MEDIA_TYPE_VIDEO, 90000));
-  fail_unless (fs_session_set_codec_preferences (session, codecs,
+  fail_unless (fs_stream_set_remote_codecs (stream, codecs,
           &error),
-      "Unable to set codec preferences: %s",
+      "Unable to set remote codec: %s",
       error ? error->message : "UNKNOWN");
 
   fs_codec_list_destroy (codecs);
 
 
-  pipeline = gst_parse_launch ("videotestsrc is-live=1 num-buffers=50 !"
+  pipeline = gst_parse_launch (
+      "videotestsrc is-live=1 num-buffers="G_STRINGIFY (BUFFER_COUNT)" !"
       " video/x-raw-yuv, framerate=(fraction)30/1 ! theoraenc !"
-      " rtptheorapay name=pay ! application/x-rtp, payload=96 !"
+      " rtptheorapay name=pay config-interval=1 name=pay !"
+      " application/x-rtp, payload=96, ssrc=(uint)12345678 !"
       " udpsink host=127.0.0.1 name=sink", NULL);
 
   gst_element_set_state (fspipeline, GST_STATE_PLAYING);
@@ -182,6 +270,39 @@ GST_START_TEST (test_rtprecv_no_config_data)
   gst_message_unref (msg);
   gst_object_unref (bus);
 
+
+  g_mutex_lock (count_mutex);
+  while (buffer_count < BUFFER_COUNT)
+    g_cond_wait (count_cond, count_mutex);
+  buffer_count = 0;
+  g_mutex_unlock (count_mutex);
+
+  gst_element_set_state (pipeline, GST_STATE_NULL);
+
+  pay = gst_bin_get_by_name (GST_BIN (pipeline), "pay");
+  ts_fail_unless (pay != NULL);
+  pad = gst_element_get_static_pad (pay, "src");
+  ts_fail_unless (pad != NULL);
+  g_signal_connect (pad, "notify::caps", G_CALLBACK (caps_changed), stream);
+  caps_changed (pad, NULL, stream);
+  gst_object_unref (pad);
+  gst_object_unref (pay);
+
+  gst_element_set_state (pipeline, GST_STATE_PLAYING);
+
+  bus = gst_element_get_bus (pipeline);
+  msg = gst_bus_timed_pop_filtered (bus, GST_CLOCK_TIME_NONE, GST_MESSAGE_EOS);
+  gst_message_unref (msg);
+  gst_object_unref (bus);
+
+  g_mutex_lock (count_mutex);
+  while (buffer_count < BUFFER_COUNT)
+    g_cond_wait (count_cond, count_mutex);
+  buffer_count = 0;
+  g_mutex_unlock (count_mutex);
+
+  gst_element_set_state (pipeline, GST_STATE_NULL);
+
   bus = gst_element_get_bus (fspipeline);
   msg = gst_bus_pop_filtered (bus, GST_MESSAGE_ERROR);
   if (msg)
@@ -199,7 +320,6 @@ GST_START_TEST (test_rtprecv_no_config_data)
   }
   gst_object_unref (bus);
 
-  gst_element_set_state (pipeline, GST_STATE_NULL);
 
   gst_object_unref (pipeline);
   gst_object_unref (participant);
@@ -208,9 +328,10 @@ GST_START_TEST (test_rtprecv_no_config_data)
 
   gst_element_set_state (fspipeline, GST_STATE_NULL);
 
-
-
   gst_object_unref (fspipeline);
+
+  g_mutex_free (count_mutex);
+  g_cond_free (count_cond);
 }
 GST_END_TEST;
 
@@ -226,8 +347,8 @@ fsrtprecvcodecs_suite (void)
   fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
   g_log_set_always_fatal (fatal_mask);
 
-  tc_chain = tcase_create ("fsrtprecv_no_config_data");
-  tcase_add_test (tc_chain, test_rtprecv_no_config_data);
+  tc_chain = tcase_create ("fsrtprecv_inband_config_data");
+  tcase_add_test (tc_chain, test_rtprecv_inband_config_data);
   suite_add_tcase (s, tc_chain);
 
   return s;
