@@ -707,12 +707,9 @@ _create_sinksource (gchar *elementname, GstBin *bin,
   g_object_set (elem,
     "closefd", FALSE,
     "sockfd", fd,
+    "auto-multicast", FALSE,
     NULL);
 
-  if (g_object_class_find_property (G_OBJECT_GET_CLASS (elem),
-          "auto-multicast"))
-    g_object_set (elem, "auto-multicast", FALSE, NULL);
-
   if (!gst_bin_add (bin, elem)) {
     g_set_error (error, FS_ERROR, FS_ERROR_CONSTRUCTION,
       "Could not add the %s element to the gst %s bin", elementname,
